DT1_TOKEN_CONFIG_KEY = "token_dt1"
CONFIG_DOCKER_USERNAME = "docker_username"
CONFIG_DUCKIETOWN_VERSION = "duckietown_version"

CONFIG_DOCKER_PASSWORD = "docker_password"

IMPORTANT_ENVS = {
    "AIDO_REGISTRY": "docker.io",
    "PIP_INDEX_URL": "https://pypi.org/simple",
    "DTSERVER": "https://challenges.duckietown.org/v4",
}
